#!/usr/bin/env python
# -*- coding: utf-8 -*-

#===============================================================================
#
#
#   THIS FILE IS AUTOGENERATED DO NOT MODIFY
#
#
#===============================================================================

# Copyright 2012 Liricus SRL <info@liricus.com.ar>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Fra


#===============================================================================
# IMPORTS
#===============================================================================

import unittest
import os
import json
import random
import hashlib
import tempfile
import datetime
import time
import atexit
import shutil

import peewee

from flaskext import uploads


#===============================================================================
# IMPORTS AND FUNCTIONS TO CONFIGURE SOCIALFRAME SERVER
#===============================================================================


import settings as user_settings
FD, DB = tempfile.mkstemp("_socialframe_test.db")
user_settings.HOST = "localhost"
user_settings.PORT = 8080
user_settings.DB = {
    "schema": "sqlite3",
    "name": DB,
}
user_settings.IMAGES = tempfile.mkdtemp()
user_settings.TESTING = True
user_settings.USER_NEVER_ACTIVE_EXPIRATION = 0
user_settings.USER_UNLOGED_EXPIRATION = 0
user_settings.USER_SUSPENDED_EXPIRATION = 0
user_settings.BUBBLE_INVITATION_EXPIRATION = 0

import socialframe_server
socialframe_server.init_settings(user_settings)

from socialframe_server import models, core, jobs, settings
from socialframe_server import fileutils
from socialframe_server.ext import facebook

import test_data


#===============================================================================
# CONSTANTS
#===============================================================================

PATH = os.path.abspath(os.path.dirname(__file__))

TEST_APP = core.APP.test_client()


#===============================================================================
# FUNCTIONS
#===============================================================================

def mutter(f):
    return None


def testif(cond):

    def _wrap(func):
        if cond:
            return func
        return None

    return _wrap


#===============================================================================
# BASE TESTS
#===============================================================================

class SocialFrameBaseTest(unittest.TestCase):

    def full_url(self, action, args):
        return "/" + "/".join([action] + list(args))

    def giveme_user(self, password=None, email=None, is_superuser=False,
                     is_enterprise=False, status=models.User.STATUS_ACTIVE):
        user = models.User()
        user.email = self.email if email is None else email
        user.birth = datetime.datetime.now().date()
        user.is_enterprise = is_enterprise
        user.sex = "m" if random.randint(0, 1) else "f"
        user.is_superuser = is_superuser
        user.fname = str(random.random())
        user.lname = str(random.random())
        user.last_login = datetime.datetime.now()
        user.set_status(status)
        user.set_password(password or str(random.random()))
        user.save()
        return user

    def call(self, action, args=(), data={}, error={}, parse=True, method="GET"):
        url = self.full_url(action, args)
        if method == "GET":
            method = TEST_APP.get
        elif method == "POST":
            method = TEST_APP.post
        elif method == "PUT":
            method = TEST_APP.put
        response = method(url, data=data)
        if not parse:
            return response
        data = json.loads(response.data)
        if data["action"] == "index":
            self.assertEquals("/", url)
        else:
            if "/" in action:
                action = action.rsplit("/", 1)[-1]
            self.assertIn(data["action"], [action, None])
        msg = u"url {} - data {}".format(url, str(data))
        if error:
            self.assertEqual(data["error"], True)
            for k, v in error.items():
                self.assertEqual(data[k], v, msg=msg)
        else:
            self.assertNotEqual(data["error"], True, msg=msg)
        return data

    def setUp(self):
        self._unique = random.random() + time.time()
        self.email = "test@socialframe.com"

    def unique_name(self):
        self._unique += 1
        return str(self._unique)

    def tearDown(self):
        try:
            self.call("logout")
        except:
            pass
        for model in models.autocreatetables():
            for instance in model.select():
                instance.delete_instance(recursive=True)


#===============================================================================
# TEST
#===============================================================================

class APITest(SocialFrameBaseTest):

    def test_index(self):
        self.call("")

    def test_api_help(self):
        self.call("apihelp")

    def test_create_user(self):
        data = {}
        data["email"] = self.email
        data["password"] = str(random.random())
        data["fname"] = str(random.random())
        data["lname"] = str(random.random())
        data["sex"] = "m" if random.randint(0, 1) else "f"
        data["birth"] = datetime.datetime.now().strftime("%Y%m%d")

        self.call(
            "register_user", data=data, method="POST"
        )
        user = models.User.get(models.User.email == self.email)
        self.assertFalse(user.active)
        profile = models.Profile.get(models.Profile.user == user)
        self.assertEquals(profile.operation, models.Profile.OPERATION_CREATE_USER)
        self.call(
            "confirm_registration", (self.email, profile.operation_hash)
        )
        user = models.User.get(models.User.email == self.email)
        self.assertTrue(user.active)

    def test_login_logout(self):
        password = str(random.random())
        user = self.giveme_user(password)
        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST")
        data = self.call("is_loggedin")
        self.assertTrue(data["is_loggedin"])
        self.call("logout")

    def test_reset_password(self):
        password = str(random.random())
        user = self.giveme_user(password)
        self.assertTrue(user.check_password(password))
        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST")
        self.call("reset_password", data={"email": self.email}, method="POST")
        user = models.User.get(models.User.email == self.email)
        self.assertFalse(user.check_password(password))
        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST", error={"code": 403})

    def test_delete_user(self):
        password = str(random.random())
        user = self.giveme_user(password)
        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST")
        self.call("delete_user", data={"password": password}, method="POST")
        user = models.User.get(models.User.email == self.email)
        profile = models.Profile.get(models.Profile.user == user)
        self.assertEquals(profile.operation,
                          models.Profile.OPERATION_DELETE_USER)
        self.call("confirm_delete", (self.email, profile.operation_hash))
        with self.assertRaises(peewee.DoesNotExist):
            user = models.User.get(models.User.email == self.email)
            user.profile

    def test_crud_bubles(self):
        password = str(random.random())
        user = self.giveme_user(password)
        for listed in (True, False):
            for bubble_password in (True, False):
                for tags in (True, False):
                    for desc in (True, False):
                        bubble_name = str(random.random())

                        if listed:
                            listed = "true"
                        else:
                            listed = "false"

                        if desc:
                            desc = str(random.random())
                        else:
                            desc = "null"

                        if bubble_password:
                            bubble_password = str(random.random())
                        else:
                            bubble_password = "null"

                        if tags:
                            tags = ",".join([
                                hashlib.sha1(str(random.random())).hexdigest()
                                for c in range(random.randint(1, 10))
                            ])
                        else:
                            tags = "null"
                        data = {}
                        data["bubble_name"] = bubble_name
                        data["listed"] = listed
                        data["desc"] = desc
                        data["tags"] = tags
                        data["type"] = "private"
                        data["password"] = bubble_password
                        self.call(
                            "create_bubble", data=data, error={"code": 403}, method="POST"
                        )

                        self.call("login",
                                  data={"email": user.email,
                                        "password": password},
                                  method="POST")
                        self.call("create_bubble", data=data, method="POST")
                        data = self.call("list_bubbles", (user.email,))

                        if listed == "true":
                            self.assertIn(bubble_name, data["bubbles"])
                            bubble_data = data["bubbles"][bubble_name]
                            if tags != "null":
                                for tag in tags.split(","):
                                    self.assertIn(tag.strip(), bubble_data["tags"])
                            else:
                                self.assertFalse(bubble_data["tags"])
                        else:
                            self.assertNotIn(bubble_name, data["bubbles"])

                        data = self.call("my_bubbles")

                        self.assertIn(bubble_name, data["bubbles"])
                        bubble_data = data["bubbles"][bubble_name]
                        if tags != "null":
                            for tag in tags.split(","):
                                self.assertIn(tag.strip(), bubble_data["tags"])
                        else:
                            self.assertFalse(bubble_data["tags"])

                        if bubble_password != "null":
                            self.call(
                                "list_photos", (user.email, bubble_name, "null"),
                                error={"code": 403}
                            )

                        self.call("delete_bubble",
                                  data={"bubble_name": bubble_name}, method="POST")

                        data = self.call("list_bubbles", (user.email,))
                        self.assertNotIn(bubble_name, data["bubbles"])

                        self.call(
                            "listphotos", (user.email, bubble_name),
                            error={"code": 404}
                        )

                        self.call("logout")

    def test_crud_image(self):
        password = str(random.random())
        user = self.giveme_user(password)

        photos = list(test_data.ls())
        hashes = {}
        for photo in photos:
            #upload_name = uploads.secure_filename(photo)
            hashed_name = ".".join([
                hashlib.sha1(photo).hexdigest(), photo.rsplit(".", 1)[1]
            ])
            hashes[hashed_name] = {
                "sha": fileutils.sha(photo)
            }
        bubbles = []
        with core.APP_DB.database.transaction():
            for bubble_password in (True, False):
                bubble = models.Bubble()
                bubble.user = user
                bubble.name = self.unique_name()
                bubble.listed = bool(random.randint(0, 1))
                bubble.type = models.Bubble.PRIVATE_BUBBLE
                if bubble_password:
                    bubble_password = str(random.random())
                    bubble.set_password(bubble_password)
                else:
                    bubble_password = "null"
                bubble.save()
                bubbles.append((bubble, bubble_password))

        for bubble, bubble_password in bubbles:
            for photo in test_data.ls():
                self.call("upload_photo",
                          data={"bubble_name":bubble.name,
                                "photo":open(photo),
                                "email":user.email,
                                "desc": self.unique_name()},
                          method="POST", error={"code": 403})

            data = self.call("list_photos",
                             (user.email, bubble.name, bubble_password))
            self.assertFalse(data["photos"])

            self.call("login",
                      data={"email": user.email, "password": password},
                      method="POST")

            for photo in photos:
                self.call("upload_photo",
                          data={"bubble_name":bubble.name,
                                "photo":open(photo),
                                "email":user.email,
                                "desc": self.unique_name()},
                          method="POST")

            data = self.call("list_photos",
                             (user.email, bubble.name, bubble_password))
            self.assertEquals(len(data["photos"]), len(photos))

            for pname, pdata in data["photos"].items():
                for size in settings.IMAGE_SIZES.keys():
                    url = self.full_url("get_photo",
                                        (self.email, bubble.name,
                                         pname, size, bubble_password))
                    response = TEST_APP.get(url)
                    self.assertTrue(response.mimetype.startswith("image/"))

            for pname in data["photos"].keys():
                data = self.call("delete_photo",
                                 data={"bubble_name": bubble.name,
                                       "filename": pname},
                                 method="POST")
                subdata = self.call("list_photos",
                                    (user.email, bubble.name, bubble_password))
                self.assertNotIn(pname, subdata["photos"])

            self.call("logout")

    def test_giveme_enterprise_bubbles(self):
        password = str(random.random())
        user = self.giveme_user(password)
        tags = [
            hashlib.sha1(str(random.random())).hexdigest()
            for c in range(100)
        ]
        with core.APP_DB.database.transaction():
            for bubble_password in (True, False):
                for bubble_listed in (True, False):
                    bubble = models.Bubble()
                    bubble.user = user
                    bubble.name = self.unique_name()
                    bubble.listed = bubble_listed
                    bubble.type = models.Bubble.ENTERPRISE_BUBBLE
                    if bubble_password:
                        bubble_password = str(random.random())
                        bubble.set_password(bubble_password)
                    else:
                        bubble_password = "null"
                    bubble.save()
                    if random.randint(0, 100) < 80:
                        selected_tags = set()
                        for idx in range(len(tags)):
                            selected_tags.add(random.choice(tags))
                        bubble.set_tags(selected_tags)
        data = self.call("give_me_enterprise_bubbles")
        for bdata in data["bubbles"].values():
            self.assertEqual(bdata["type"], models.Bubble.ENTERPRISE_BUBBLE)

    def test_giveme_bubbles(self):
        password = str(random.random())
        user = self.giveme_user(password)
        tags = [
            hashlib.sha1(str(random.random())).hexdigest()
            for c in range(100)
        ]
        with core.APP_DB.database.transaction():
            for bubble_password in (True, False):
                for bubble_listed in (True, False):
                    bubble = models.Bubble()
                    bubble.user = user
                    bubble.name = self.unique_name()
                    bubble.listed = bubble_listed
                    bubble.type = models.Bubble.COMMUNITY_BUBBLE
                    if bubble_password:
                        bubble_password = str(random.random())
                        bubble.set_password(bubble_password)
                    else:
                        bubble_password = "null"
                    bubble.save()
                    if random.randint(0, 100) < 80:
                        selected_tags = set()
                        for idx in range(len(tags)):
                            selected_tags.add(random.choice(tags))
                        bubble.set_tags(selected_tags)
        data = self.call("give_me_bubbles")
        for bdata in data["bubbles"].values():
            self.assertTrue(bdata["listed"])

    def test_find(self):
        password = str(random.random())
        user = self.giveme_user(password)
        data = self.call("find", ("users", user.email))
        self.assertIn(self.email, data["users"])
        bubble = models.Bubble()
        bubble.user = user
        bubble.name = self.unique_name()
        bubble.type = models.Bubble.PRIVATE_BUBBLE
        bubble.listed = True
        bubble.save()
        data = self.call("find", ("bubbles", bubble.name))
        self.assertIn(bubble.name, data["bubbles"])
        bubble.type = models.Bubble.ENTERPRISE_BUBBLE
        bubble.save()
        data = self.call("find", ("ebubbles", bubble.name))
        self.assertIn(bubble.name, data["ebubbles"])

    def test_sizes(self):
        sizes = sorted(self.call("sizes")["sizes"].items())
        self.assertEquals(sizes, sorted(settings.IMAGE_SIZES.items()))

    def test_change_user(self):
        password = str(random.random())
        user = self.giveme_user(password)
        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST")
        data = {}
        for k in ("fname", "lname", "password"):
            new_value = hashlib.sha512(str(random.random())).hexdigest()
            data[k] = new_value
        self.call("change_user", data=data, method="POST")
        user = models.User.get(models.User.email == self.email)
        for k in ("fname", "lname", "password"):
            if k == "password":
                user.check_password(data[k])
            else:
                self.assertEquals(getattr(user, k), data[k])

    def test_change_bubbles(self):
        password = str(random.random())
        user = self.giveme_user(password)
        bubbles = []
        with core.APP_DB.database.transaction():
            for bubble_password in (True, False):
                bubble = models.Bubble()
                bubble.user = user
                bubble.listed = bool(random.randint(0, 1))
                bubble.name = self.unique_name()
                bubble.type = models.Bubble.COMMUNITY_BUBBLE
                if bubble_password:
                    bubble_password = str(random.random())
                    bubble.set_password(bubble_password)
                else:
                    bubble_password = "null"
                bubble.save()
                bubbles.append(bubble.name)
        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST")
        names = []
        passwords = []
        for bname in bubbles:
            values = {"bubble_name": bname}
            for k in ("new_bubble_name", "password", "tags"):
                new_value = hashlib.sha512(str(random.random())).hexdigest()
                if k != "tags":
                    values[k] = new_value
                else:
                    values[k] = ",".join([new_value + str(idx) for idx in range(10)])
                if k == "new_bubble_name":
                    names.append(new_value)
                elif k == "password":
                    passwords.append(new_value)
            self.call("change_bubble", data=values, method="POST")
        for b in models.Bubble.select():
            self.assertIn(b.name, names)
            self.assertTrue(any(map(b.check_password, passwords)))

    def test_reactivate_user(self):
        password = str(random.random())
        user = self.giveme_user(password, status=models.User.STATUS_SUSPENDED)
        self.assertFalse(user.active)
        profile = models.Profile.get(models.Profile.user == user)
        profile.operation = models.Profile.OPERATION_REACTIVATE_USER
        profile.save()
        self.call("reactivate_user", (self.email, profile.operation_hash))
        user = models.User.get(models.User.id == user.id)
        self.assertTrue(user.active)

    def test_validate_bubble_password(self):
        user = self.giveme_user()
        bubbles = []
        with core.APP_DB.database.transaction():
            for bubble_password in (True, False):
                bubble = models.Bubble()
                bubble.user = user
                bubble.name = self.unique_name()
                bubble.listed = bool(random.randint(0, 1))
                if bubble_password:
                    bubble_password = str(random.random())
                    bubble.set_password(bubble_password)
                else:
                    bubble_password = "null"
                bubble.type = models.Bubble.COMMUNITY_BUBBLE
                bubble.save()
                bubbles.append((bubble, bubble_password))
        for bubble, bubble_password in bubbles:
            args = (self.email, bubble.name, bubble_password)
            vp = self.call("validate_bubble_password", args)["valid_password"]
            self.assertTrue(vp)
            args = (self.email, bubble.name, self.unique_name())
            vp = self.call("validate_bubble_password", args)["valid_password"]
            self.assertFalse(vp)

    def test_invite_bubble(self):
        password = str(random.random())
        user = self.giveme_user(password)
        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST")
        bubble = models.Bubble()
        bubble.user = user
        bubble.name = self.unique_name()
        bubble.listed = bool(random.randint(0, 1))
        bubble.type = models.Bubble.COMMUNITY_BUBBLE
        bubble.save()
        emails = []
        for idx in range(random.randint(1, 10)):
            emails.append("email_{}@host.con".format(idx))
        data = {"emails": ",".join(emails), "bubble_name":bubble.name}
        self.call("invite_to_bubble", data=data, method="POST")
        self.assertTrue(models.BubbleInvitation.select().count() > 0)
        for inv in models.BubbleInvitation.select():
            self.assertIn(inv.email, emails)
            self.assertEquals(inv.bubble, bubble)
            self.assertEquals(inv.sended_by, user)
            self.assertFalse(inv.sended)

    def test_join_bubble(self):
        password = str(random.random())
        user = self.giveme_user(password)
        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST")
        bubble = models.Bubble()
        bubble.user = user
        bubble.name = self.unique_name()
        bubble.listed = bool(random.randint(0, 1))
        bubble.type = models.Bubble.COMMUNITY_BUBBLE
        bubble.save()
        inv = models.BubbleInvitation()
        inv.sended_by = user
        inv.to_user = user
        inv.email = self.email
        inv.bubble = bubble
        inv.save()
        args = (inv.sended_by.email, inv.bubble.name, inv.invitation_hash)
        self.call("join_bubble", args)
        count = models.BubbleInvitation.select().where(models.BubbleInvitation.id == inv.id).count()
        self.assertEquals(count, 0)
        bc = models.BubbleCommiter.get(models.BubbleCommiter.user == user,
                                       models.BubbleCommiter.bubble == bubble)

    def test_link_bubble_invitation_on_create(self):
        user = self.giveme_user()
        bubble = models.Bubble()
        bubble.user = user
        bubble.name = self.unique_name()
        bubble.listed = bool(random.randint(0, 1))
        bubble.type = models.Bubble.COMMUNITY_BUBBLE
        bubble.save()
        inv = models.BubbleInvitation()
        inv.sended_by = user
        inv.email = "toto@toto.com"
        inv.bubble = bubble
        inv.save()
        self.assertEqual(models.BubbleInvitation.get(models.BubbleInvitation.id == inv.id).to_user, None)
        user2 = self.giveme_user(email=inv.email)
        self.assertEqual(models.BubbleInvitation.get(models.BubbleInvitation.id == inv.id).to_user, user2)

    def test_delete_invite_bubble(self):
        password = str(random.random())
        user = self.giveme_user(password)
        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST")
        bubble = models.Bubble()
        bubble.user = user
        bubble.name = self.unique_name()
        bubble.listed = bool(random.randint(0, 1))
        bubble.type = models.Bubble.COMMUNITY_BUBBLE
        bubble.save()
        inv = models.BubbleInvitation()
        inv.sended_by = user
        inv.email = "toto@toto.com"
        inv.bubble = bubble
        inv.save()
        data = {"emails": inv.email, "bubble_name": bubble.name}
        self.call("delete_bubble_invitation", data=data, method="POST")
        self.assertEqual(models.BubbleInvitation.select().where(models.BubbleInvitation.id == inv.id).count(), 0)

    def test_delete_bubble_commiters(self):
        password = str(random.random())
        user = self.giveme_user(password)
        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST")
        bubble = models.Bubble()
        bubble.user = user
        bubble.name = self.unique_name()
        bubble.listed = bool(random.randint(0, 1))
        bubble.type = models.Bubble.COMMUNITY_BUBBLE
        bubble.save()
        commit = models.BubbleCommiter()
        commit.user = self.giveme_user(email="toto@toto.com")
        commit.bubble = bubble
        commit.save()
        data = {"email": commit.user.email, "bubble_name": bubble.name}
        self.call("delete_bubble_commiters", data=data, method="POST")
        self.assertEqual(models.BubbleCommiter.select().where(models.BubbleCommiter.id == commit.id).count(), 0)

    def test_list_bubble_invitations(self):
        password = str(random.random())
        user = self.giveme_user(password)
        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST")
        bubble = models.Bubble()
        bubble.user = user
        bubble.name = self.unique_name()
        bubble.listed = bool(random.randint(0, 1))
        bubble.type = models.Bubble.COMMUNITY_BUBBLE
        bubble.save()
        inv = models.BubbleInvitation()
        inv.sended_by = user
        inv.sended = bool(random.randint(0, 1))
        inv.email = "toto@toto.com"
        inv.bubble = bubble
        if bool(random.randint(0, 1)):
            inv.user = self.giveme_user(email="toto@toto.com")
        inv.save()
        data = self.call("list_bubble_invitations", (bubble.name,))
        for invd in data["invitations"]:
            self.assertEquals(invd["is_user"], (inv.to_user != None))
            self.assertEquals(invd["sended"], inv.sended)
            self.assertEquals(invd["email"], inv.email)
            self.assertEquals(invd["created"],
                              core.formatdate(inv.creation_datetime))

    def test_list_bubble_commiters(self):
        password = str(random.random())
        user = self.giveme_user(password)
        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST")
        bubble = models.Bubble()
        bubble.user = user
        bubble.name = self.unique_name()
        bubble.listed = bool(random.randint(0, 1))
        bubble.type = models.Bubble.COMMUNITY_BUBBLE
        bubble.save()
        commit = models.BubbleCommiter()
        commit.user = self.giveme_user(email="toto@toto.com")
        commit.bubble = bubble
        commit.save()
        data = self.call("list_bubble_commiters", (bubble.name,))
        for cid in data["commiters"]:
            self.assertEquals(cid["email"], commit.user.email)
            self.assertEquals(cid["joined"],
                              core.formatdate(commit.joined_datetime))

    def test_my_bubbles_invitations(self):

        sended_by = self.giveme_user(email="toto@toto.com")

        bubble = models.Bubble()
        bubble.user = sended_by
        bubble.name = self.unique_name()
        bubble.listed = bool(random.randint(0, 1))
        bubble.type = models.Bubble.COMMUNITY_BUBBLE
        bubble.save()

        password = str(random.random())
        to_user = self.giveme_user(password)

        inv = models.BubbleInvitation()
        inv.sended_by = sended_by
        inv.sended = bool(random.randint(0, 1))
        inv.email = to_user.email
        inv.to_user = to_user
        inv.bubble = bubble
        inv.save()

        self.call("login",
                  data={"email": to_user.email, "password": password},
                  method="POST")

        data = self.call("my_bubbles_invitations")
        for invd in data["invitations"]:
            self.assertEquals(invd["is_user"], (inv.to_user != None))
            self.assertEquals(invd["sended"], inv.sended)
            self.assertEquals(invd["email"], inv.email)
            self.assertEquals(invd["created"],
                              core.formatdate(inv.creation_datetime))

    def test_upload_photo_as_commiter(self):
        photo = list(test_data.ls())[0]

        password = str(random.random())
        user = self.giveme_user(password)

        bubble = models.Bubble()
        bubble.user = self.giveme_user(email="toto@toto.com")
        bubble.name = self.unique_name()
        bubble.listed = bool(random.randint(0, 1))
        bubble.type = models.Bubble.COMMUNITY_BUBBLE
        bubble.save()

        commit = models.BubbleCommiter()
        commit.user = user
        commit.bubble = bubble
        commit.save()
        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST")
        self.call("upload_photo",
                   data={"bubble_name":bubble.name,
                         "photo":open(photo),
                         "email":bubble.user.email,
                         "desc": self.unique_name()},
                  method="POST")

    def test_quick_register(self):
        password = str(random.random())
        data = {}
        data["email"] = self.email
        data["password"] = password
        self.call("quick_register", data=data, method="POST")
        user = models.User.get(models.User.email == self.email)
        self.assertEqual(self.email, user.email)
        self.assertEqual(self.email, user.fname)
        self.assertEqual(self.email, user.lname)
        self.assertTrue(user.check_password(password))

    def test_set_enterprise_user(self):
        password = str(random.random())
        user = self.giveme_user(password=password)
        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST")
        self.call("set_enterprise_user", data={"email": user.email},
                  method="POST",
                  error={"code": 403})
        self.call("logout")
        suser = self.giveme_user(email="super@super.com",
                                 password=password, is_superuser=True)
        self.call("login",
                  data={"email": suser.email, "password": password},
                  method="POST")
        self.assertTrue(not models.User.get(models.User.id == user.id).is_enterprise)
        self.call("set_enterprise_user",
                  data={"email": user.email, "is_enterprise": "true"},
                  method="POST")
        self.assertTrue(models.User.get(models.User.id == user.id).is_enterprise)

    def test_upload_photo_to_enterprise(self):
        photo = list(test_data.ls())[0]

        password = str(random.random())
        user = self.giveme_user(password)

        bubble = models.Bubble()
        bubble.user = self.giveme_user(email="toto@toto.com",
                                       password=password,
                                       is_enterprise=True)
        bubble.name = self.unique_name()
        bubble.listed = bool(random.randint(0, 1))
        bubble.type = models.Bubble.ENTERPRISE_BUBBLE
        bubble.save()

        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST")
        data = self.call("upload_photo",
                   data={"bubble_name":bubble.name,
                         "photo":open(photo),
                         "email":bubble.user.email,
                         "desc": self.unique_name()},
                  method="POST")
        self.assertEqual(data["photo_status"], models.Image.STATUS_PENDING)
        data = self.call("list_photos",
                          (bubble.user.email, bubble.name, "null"))
        self.assertEquals(len(data["photos"]), 0)
        self.call("logout")

        self.call("login",
                  data={"email": bubble.user.email, "password": password},
                  method="POST")
        data = self.call("list_photos_eb", (bubble.name,))
        self.assertEqual(len(data["photos"]), 1)
        for pname, pdata in data["photos"].items():
                for size in settings.IMAGE_SIZES.keys():
                    url = self.full_url("get_photo",
                                        (bubble.user.email,
                                        bubble.name, pname, size, "null"))
                    response = TEST_APP.get(url)
                    self.assertTrue(json.loads(response.data)["error"])

                    url = self.full_url("get_photo_eb",
                                        (bubble.name, pname, size))
                    response = TEST_APP.get(url)
                    self.assertTrue(response.mimetype.startswith("image/"))

    def test_create_and_change_enterprise_bubble(self):
        password = str(random.random())
        user = self.giveme_user(password)
        self.call("login",
                  data={"email": user.email,
                        "password": password},
                  method="POST")
        data = {}
        data["bubble_name"] = self.unique_name()
        data["listed"] = bool(random.randint(0, 1))
        data["desc"] = "desc"
        data["tags"] = "null"
        data["type"] = models.Bubble.ENTERPRISE_BUBBLE
        data["password"] = self.unique_name()
        self.call("create_bubble", data=data,
                  error={"code": 403}, method="POST")

        bubble_tomod = models.Bubble()
        bubble_tomod.user = user
        bubble_tomod.name = self.unique_name()
        bubble_tomod.listed = bool(random.randint(0, 1))
        bubble_tomod.type = models.Bubble.PRIVATE_BUBBLE
        bubble_tomod.save()

        self.call("create_bubble", data=data,
                  error={"code": 403}, method="POST")
        self.call("change_bubble",
                  data={"bubble_name": bubble_tomod.name,
                        "type": models.Bubble.ENTERPRISE_BUBBLE},
                  error={"code": 403},
                  method="POST")

        user.is_enterprise = True
        user.save()

        self.call("create_bubble", data=data, method="POST")
        self.call("change_bubble",
                  data={"bubble_name": bubble_tomod.name,
                        "type": models.Bubble.ENTERPRISE_BUBBLE},
                  method="POST")

    def test_user_data(self):
        password = str(random.random())
        user = self.giveme_user(password)
        self.call("login",
                  data={"email": user.email,
                        "password": password},
                  method="POST")
        user = models.User.get(models.User.email == user.email)
        data = self.call("user_data", (user.email,))
        for k, v in data["user"].items():
            uv = getattr(user, k)
            if isinstance(uv, datetime.datetime):
                uv = core.formatdatetime(uv)
            elif isinstance(uv, datetime.date):
                uv = core.formatdate(uv)
            self.assertEquals(v, uv)

    def test_change_photo(self):
        photo = list(test_data.ls())[0]

        password = str(random.random())
        user = self.giveme_user(password)

        bubble = models.Bubble()
        bubble.user = user,
        bubble.name = self.unique_name()
        bubble.listed = bool(random.randint(0, 1))
        bubble.type = models.Bubble.ENTERPRISE_BUBBLE
        bubble.save()

        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST")
        data = self.call("upload_photo",
                   data={"bubble_name":bubble.name,
                         "photo":open(photo),
                         "email":bubble.user.email,
                         "desc": self.unique_name()},
                  method="POST")
        img = models.Image.get(models.Image.bubble == bubble,
                               models.Image.name == data["filename"])
        data = {"email": bubble.user.email,
                "bubble_name": bubble.name,
                "desc": self.unique_name(),
                "filename": img.name}
        self.call("change_photo", data=data, method="POST")
        img = models.Image.get(models.Image.bubble == bubble,
                               models.Image.name == data["filename"])
        self.assertEquals(img.desc, data["desc"])

        bubble.user = self.giveme_user(email="toto@toto.com")
        bubble.save()
        data["desc"] = self.unique_name()
        data["email"] = bubble.user.email
        self.call("change_photo", data=data, method="POST")
        img = models.Image.get(models.Image.bubble == bubble,
                               models.Image.name == data["filename"])
        self.assertEquals(img.desc, data["desc"])

        bubble.type = models.Bubble.PRIVATE_BUBBLE
        bubble.save()
        img.uploaded_by = self.giveme_user(email="perico@loro.com")
        img.save()
        self.call("change_photo", data=data, method="POST", error={"code":403})

    def test_moderate_bubble(self):
        photo = list(test_data.ls())[0]

        password = str(random.random())
        user = self.giveme_user(password, is_enterprise=True)

        bubble = models.Bubble()
        bubble.user = user,
        bubble.name = self.unique_name()
        bubble.listed = bool(random.randint(0, 1))
        bubble.type = models.Bubble.ENTERPRISE_BUBBLE
        bubble.save()

        tu_password = str(random.random())
        tu_user = self.giveme_user(tu_password, email="toto@toto.com")
        self.call("login",
                  data={"email": tu_user.email, "password": tu_password},
                  method="POST")
        data = self.call("upload_photo",
                   data={"bubble_name":bubble.name,
                         "photo":open(photo),
                         "email":bubble.user.email,
                         "desc": self.unique_name()},
                  method="POST")
        self.call("logout")

        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST")

        query = models.Image.select()
        query = query.where(models.Image.bubble == bubble,
                            models.Image.name == data["filename"])
        q_pend = query.where(models.Image.status == models.Image.STATUS_PENDING)
        q_app = query.where(models.Image.status == models.Image.STATUS_APPROVED)

        self.assertEquals(q_pend.count(), 1)
        self.assertEquals(q_app.count(), 0)

        self.call("moderate_bubble",
                  data={"bubble_name": bubble.name,
                        "to_approve": data["filename"] + ",",
                        "to_delete": ","},
                    method="POST")
        self.assertEquals(q_pend.count(), 0)
        self.assertEquals(q_app.count(), 1)

        self.call("moderate_bubble",
                  data={"bubble_name": bubble.name,
                        "to_delete": data["filename"] + ",",
                        "to_approve": ","},
                    method="POST")
        self.assertEquals(q_pend.count(), 0)
        self.assertEquals(q_app.count(), 0)


#===============================================================================
# CRON TEST
#===============================================================================

class JobsTest(SocialFrameBaseTest):

    def test_delete_inactive_users(self):
        user = self.giveme_user(status=models.User.STATUS_INACTIVE)
        self.assertEquals(models.User.select().where(models.User.id == user.id).count(), 1)
        jobs.delete_inactive_users()
        self.assertEquals(models.User.select().where(models.User.id == user.id).count(), 0)
        user = self.giveme_user()
        self.assertEquals(models.User.select().where(models.User.id == user.id).count(), 1)
        jobs.delete_inactive_users()
        self.assertEquals(models.User.select().where(models.User.id == user.id).count(), 1)

    def test_suspend_user(self):
        user = self.giveme_user(status=models.User.STATUS_ACTIVE)
        query = models.User.select().where(models.User.id == user.id,
                                    models.User.status == models.User.STATUS_SUSPENDED)
        self.assertEquals(query.count(), 0)
        jobs.suspend_user()
        self.assertEquals(query.count(), 1)

    def test_delete_suspended_user(self):
        user = self.giveme_user(status=models.User.STATUS_SUSPENDED)
        query = models.User.select().where(models.User.id == user.id,
                                    models.User.status == models.User.STATUS_SUSPENDED)
        self.assertEquals(query.count(), 1)
        jobs.delete_suspended_user()
        self.assertEquals(query.count(), 0)
        user = self.giveme_user()
        query = models.User.select().where(models.User.id == user.id)
        self.assertEquals(query.count(), 1)
        jobs.delete_suspended_user()
        self.assertEquals(query.count(), 1)

    def test_send_bubble_invitations(self):
        user = self.giveme_user(status=models.User.STATUS_ACTIVE)
        bubble = models.Bubble()
        bubble.user = user
        bubble.name = self.unique_name()
        bubble.listed = bool(random.randint(0, 1))
        bubble.type = models.Bubble.COMMUNITY_BUBBLE
        bubble.save()
        inv = models.BubbleInvitation()
        inv.sended_by = user
        inv.email = self.email
        inv.bubble = bubble
        inv.save()
        self.assertFalse(inv.sended)
        jobs.send_bubble_invitations()
        inv = models.BubbleInvitation.get(models.BubbleInvitation.id == inv.id)
        self.assertTrue(inv.sended)

    def test_delete_expired_bubble_invitations(self):
        user = self.giveme_user()
        bubble = models.Bubble()
        bubble.user = user
        bubble.name = self.unique_name()
        bubble.listed = bool(random.randint(0, 1))
        bubble.type = models.Bubble.COMMUNITY_BUBBLE
        bubble.save()
        inv = models.BubbleInvitation()
        inv.sended_by = user
        inv.email = self.email
        inv.bubble = bubble
        inv.save()
        jobs.delete_expired_bubble_invitations()
        count = models.BubbleInvitation.select().where(models.BubbleInvitation.id == inv.id).count()
        self.assertEquals(count, 0)

    def test_delete_sended_bubble_invitation(self):
        expiration = settings.HASH_EXPIRATION * 10
        user = self.giveme_user()
        bubble = models.Bubble()
        bubble.user = user
        bubble.name = self.unique_name()
        bubble.listed = bool(random.randint(0, 1))
        bubble.type = models.Bubble.COMMUNITY_BUBBLE
        bubble.save()
        inv = models.BubbleInvitation()
        inv.creation_datetime = (datetime.datetime.now()
                                 - datetime.timedelta(seconds=expiration))
        inv.sended_by = user
        inv.email = self.email
        inv.bubble = bubble
        inv.save()
        jobs.delete_sended_bubble_invitation()
        self.assertEquals(models.BubbleInvitation.select().where(models.BubbleInvitation.id == inv.id).count(), 1)
        inv.sended = True
        inv.save()
        jobs.delete_sended_bubble_invitation()
        self.assertEquals(models.BubbleInvitation.select().where(models.BubbleInvitation.id == inv.id).count(), 0)


#===============================================================================
# EXT
#===============================================================================

@testif(settings.FACEBOOK and settings.FACEBOOK_DEBUG_ACCESS_TOKEN)
class FacebookTest(SocialFrameBaseTest):

    def get_fb(self, user):
        import pyfb
        fuser = facebook.FBUser.get(facebook.FBUser.user == user)
        fb = pyfb.Pyfb(settings.FACEBOOK_APP_ID)
        fb.set_access_token(fuser.auth_token)
        return fb

    def test_authenticate(self):
        redirect = str(random.random())
        password = str(random.random())
        user = self.giveme_user(password)
        self.call("login",
                  data={"email": user.email,
                        "password": password},
                  method="POST")
        response = self.call("facebook/authenticate",
                             data={"redirect": redirect},
                             parse=False,
                             method="POST")
        self.assertEqual(response.status_code, 302)
        fuser = facebook.FBUser.get(facebook.FBUser.user == user)
        self.assertEquals(fuser.redirect, redirect)

    def test_is_authenticated(self):
        redirect = str(random.random())
        password = str(random.random())
        user = self.giveme_user(password)
        self.call("login",
                  data={"email": user.email,
                        "password": password},
                  method="POST")
        self.call("facebook/authenticate",
                  data={"redirect": redirect},
                         parse=False,
                         method="POST")
        data = self.call("facebook/is_authenticated")
        self.assertFalse(data["is_authenticated"])
        fuser = facebook.FBUser.get(facebook.FBUser.user == user)
        fuser.auth_token = settings.FACEBOOK_DEBUG_ACCESS_TOKEN
        fuser.save()
        data = self.call("facebook/is_authenticated")
        self.assertTrue(data["is_authenticated"])

    def test_add_list_delete_sync(self):
        redirect = str(random.random())
        password = str(random.random())
        user = self.giveme_user(password)
        bubble = models.Bubble()
        bubble.user = user,
        bubble.name = self.unique_name()
        bubble.listed = bool(random.randint(0, 1))
        bubble.type = models.Bubble.ENTERPRISE_BUBBLE
        bubble.save()

        self.call("login",
                  data={"email": user.email,
                        "password": password},
                  method="POST")
        data = self.call("facebook/authenticate",
                  data={"redirect": redirect},
                         parse=False,
                         method="POST")
        fuser = facebook.FBUser.get(facebook.FBUser.user == user)
        fuser.auth_token = settings.FACEBOOK_DEBUG_ACCESS_TOKEN
        fuser.save()
        self.call("facebook/add_sync",
                  data={"bubble_name": bubble.name},
                  method="POST")
        sync = facebook.FBAlbum2BubbleSync.get(facebook.FBAlbum2BubbleSync.bubble == bubble)
        self.assertIn(bubble.name, sync.album_name)
        data = self.call("facebook/list_sync")
        self.assertEqual(len(data["syncs"]), 1)
        data = self.call("facebook/is_sync", (bubble.name,))
        self.assertTrue(data["is_sync"])
        self.call("facebook/delete_sync",
                  data={"bubble_name": bubble.name},
                  method="POST")
        data = self.call("facebook/list_sync")
        self.assertEqual(len(data["syncs"]), 0)

    def test_upload_photo(self):
        photo = list(test_data.ls())[0]

        password = str(random.random())
        user = self.giveme_user(password, is_enterprise=True)

        bubble = models.Bubble()
        bubble.user = user
        bubble.name = self.unique_name()
        bubble.listed = bool(random.randint(0, 1))
        bubble.type = models.Bubble.ENTERPRISE_BUBBLE
        bubble.save()

        self.call("login",
                  data={"email": user.email, "password": password},
                  method="POST")
        data = self.call("upload_photo",
                         data={"bubble_name":bubble.name,
                                "photo":open(photo),
                                "email":bubble.user.email,
                                "desc": self.unique_name()},
                         method="POST")

        for img  in bubble.image_set:
            img.status = models.Image.STATUS_APPROVED
            img.save()

        data = self.call("facebook/authenticate",
                         data={"redirect": str(random.random())},
                                parse=False,
                                method="POST")

        fuser = facebook.FBUser.get(facebook.FBUser.user == user)
        fuser.auth_token = settings.FACEBOOK_DEBUG_ACCESS_TOKEN
        fuser.save()

        self.call("facebook/add_sync",
                  data={"bubble_name": bubble.name,
                        "size": settings.SMALL_SIZE},
                  method="POST")

        self.call("facebook/sync_bubble",
                  data={"bubble_name": bubble.name},
                  method="POST")


#===============================================================================
# MAIN
#===============================================================================

if __name__ == "__main__":

    def cleanup():
        os.close(FD)
        os.remove(DB)
        shutil.rmtree(settings.IMAGES)

    atexit.register(cleanup)
    unittest.main(verbosity=2, failfast=True)
